/* 
 * Copyright 1999,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.log4j.appender;

import java.io.File;
import java.util.Iterator;

import org.apache.log4j.helpers.FileHelper;
import org.apache.log4j.helpers.LogLog;

/**
 * Responsible for periodically sampling the number of log files with a given
 * base filename and time-based extension, and for deleting the older files if
 * the minimum specified space remains available on the filesystem.
 * <p>
 * Requires Java 6 and up.
 * 
 * @author <a href="mailto:berndq AT gmx DOT net">Bernd</a>
 * @version 1.1
 */
public final class AbsoluteMinFreeDiskSpaceLogFileScavenger extends
    AbstractLogFileScavenger {

  public void scavenge() {
    // get complete sorted list of files
    final LogFileList logFileList = super.logFileList();
    if (logFileList.size() == 0) {
      return;
    }
    // TODO getUsableSpace() is a Java 1.6 feature!
    final long usableDiscSpace = logFileList.firstFile().getUsableSpace();
    // log files use too much disk space?
    if (usableDiscSpace < this.getProperties().getMinFreeDiscSpace()) {
      // yes, then find out out many log files need to be deleted
      final long needToFree = this.getProperties().getMinFreeDiscSpace()
          - usableDiscSpace;
      LogLog.debug("Need to free " + needToFree
          + " bytes to get the configured minimum of "
          + this.getProperties().getMinFreeDiscSpace()
          + " bytes free disc space");
      final FileHelper fileHelper = new FileHelper();
      long freeableDiscSpace = 0L;
      // old ones are at start of list
      int lastToBeDeleted = -1;
      for (final Iterator iter = logFileList.iterator(); iter.hasNext();) {
        final File logFile = (File) iter.next();
        freeableDiscSpace += logFile.length();
        lastToBeDeleted++;
        if (freeableDiscSpace >= needToFree) {
          break;
        }
      }
      if (lastToBeDeleted == -1) {
        LogLog.debug("Should free " + needToFree
            + " bytes, but not deletable logfiles found");
        return;
      }
      if (lastToBeDeleted >= 0) {
        LogLog.debug("About to delete " + (lastToBeDeleted + 1)
            + " log file(s) which will recover " + freeableDiscSpace
            + " bytes on disk.");
      }
      long freed = 0L;
      for (int d = 0; d <= lastToBeDeleted; d++) {
        final File logFile = (File) logFileList.get(d);
        final long size = logFile.length();
        if (fileHelper.deleteExisting(logFile)) {
          LogLog.debug("Scavenged log file '" + logFile.getName()
              + "\', freed " + size + " bytes.");
          freed += size;
        }
      }
      LogLog.debug("Totally freed " + freed + " bytes.");
    } else {
      LogLog.debug("No need to scavenge log files: "
          + (usableDiscSpace - this.getProperties().getMinFreeDiscSpace())
          + " bytes of disk space available for log files");
    }
  }
}
